<?php
// -- Pippi RCON 1.1.0 -- //
function pippiDetails($rcon, $platform_id) {
	$pippi = [];

	// step 1 - list players, find char name for char id
	$result = $rcon->send('listplayers');
	$lines = explode("\n", $result);
	array_shift($lines); // strip header
	array_pop($lines); // strip tail

	// parse result one line at a time.
	$name = "";
	foreach ($lines as $line) {
		$param = explode("|", $line);
		$name = trim($param[1]); // char name
		$id = trim($param[4]);   // plattform id (steam id)

		// compare id
		if ($id == $platform_id) {
			$pippi["name"] = $name;
			break;
		}
	}

	// should have a char name by now
	if ($name == "")
		return false;

	// step 2 - fetch pippi charvars
	$result = $rcon->send('charvar list "'.$name.'"');
	$lines = explode("\n", $result);
	$charvars = [];
	if (count($lines) > 2){
		array_shift($lines); // strip header 1 (Player Name: Amunet)
		array_shift($lines); // strip header 2 (CharVar | Value)

		foreach ($lines as $line) {
			$param = explode("|", $line);
			if (count($param) > 1){
				$charvar = trim($param[0]); // CharVar
				$value = trim($param[1]); // Value
				$charvars[$charvar] = $value;
			}
		}
	}
	$pippi["charvars"] = $charvars;

	return $pippi;
}

function pippiExport($rcon, $platform_id) {
	$pippi = pippiDetails($rcon, $platform_id);
	$json_string = json_encode($pippi);
	if (!$json_string)
		return;

	// got valid json, save to file
	$filename = 'pippi_'.$platform_id.'.json';
	$handle = fopen($filename, 'w');
	fwrite($handle, $json_string);
	fclose($handle);
}

function pippiImport($rcon, $platform_id) {
	// step 1 - check if we got any file to import
	$filename = 'pippi_'.$platform_id.'.json';
	if (!file_exists($filename))
		return;

	// try to read the file
	$handle = fopen($filename, 'r');
	$json_string = fread($handle, filesize($filename));
	fclose($handle);
	$pippiSave = json_decode($json_string, true);

	// step 2 - now get the "live" stats
	$pippiLive = pippiDetails($rcon, $platform_id);

	// sanity check - compare names
	if ($pippiSave["name"] != $pippiLive["name"]) {
		return;
	}
	$name = $pippiLive["name"];

	// step 3 - compare charvars
	$charvarsLive = $pippiLive["charvars"];
	$charvarsSave = $pippiSave["charvars"];

	// check charvars NOT in save and remove
	foreach ($charvarsLive as $key => $value) {
		if (!isset($charvarsSave[$key]))
			$result = $rcon->send('charvar remove "'.$name.'" "'.$key.'"');
	}

	// check charvars NOT in live and add/update
	foreach ($charvarsSave as $key => $value) {
		if (!isset($charvarsLive[$key])) {
			$result = $rcon->send('charvar set "'.$name.'" "'.$key.'" "'.$value.'"');
		} elseif ($charvarsSave[$key] != $charvarsLive[$key]) {
			$result = $rcon->send('charvar set "'.$name.'" "'.$key.'" "'.$value.'"');
		}
	}

	// step 4 - hooray!
}
?>